@@ -45,13 +45,16 @@
 		g_value_unset (&value); \
 	} while (0)
 
-#define UPDATE_FROM_INT(type, name, format) \
+#define UPDATE_FROM_INT(type, name, format, empty) \
 	do { \
 		char *temp; \
 		bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw), \
 						 type, &value); \
-		temp = g_strdup_printf (gettext (format), \
+		if (g_value_get_int (&value) != 0) \
+			temp = g_strdup_printf (gettext (format), \
 					g_value_get_int (&value)); \
+		else \
+			temp = g_strdup (empty); \
 		bacon_video_widget_properties_set_label (props, name, temp); \
 		g_free (temp); \
 		g_value_unset (&value); \
@@ -77,7 +80,7 @@
 struct BaconVideoWidgetPropertiesPrivate
 {
 	GladeXML *xml;
-	GtkWidget *vbox;
+	int time;
 };
 
 static GtkWidgetClass *parent_class = NULL;
@@ -93,8 +96,6 @@ static void
 bacon_video_widget_properties_init (BaconVideoWidgetProperties *props)
 {
 	props->priv = g_new0 (BaconVideoWidgetPropertiesPrivate, 1);
-	props->priv->xml = NULL;
-	props->priv->vbox = NULL;
 }
 
 static void
@@ -145,31 +146,50 @@ bacon_video_widget_properties_reset (BaconVideoWidgetProperties *props)
 	/* Year */
 	bacon_video_widget_properties_set_label (props, "year", _("Unknown"));
 	/* Duration */
-	bacon_video_widget_properties_set_label (props, "duration", _("0 second"));
+	bacon_video_widget_properties_from_time (props, 0);
+
 	/* Dimensions */
-	bacon_video_widget_properties_set_label (props, "dimensions", _("0 x 0"));
+	bacon_video_widget_properties_set_label (props, "dimensions", _("N/A"));
 	/* Video Codec */
 	bacon_video_widget_properties_set_label (props, "vcodec", _("N/A"));
 	/* Video Bitrate */
 	bacon_video_widget_properties_set_label (props, "video_bitrate",
-			_("0 kbps"));
+			_("N/A"));
 	/* Framerate */
 	bacon_video_widget_properties_set_label (props, "framerate",
-			_("0 frames per second"));
+			_("N/A"));
 	/* Audio Bitrate */
 	bacon_video_widget_properties_set_label (props, "audio_bitrate",
-			_("0 kbps"));
+			_("N/A"));
 	/* Audio Codec */
 	bacon_video_widget_properties_set_label (props, "acodec", _("N/A"));
 }
 
+void
+bacon_video_widget_properties_from_time (BaconVideoWidgetProperties *props,
+					 int time)
+{
+	char *string;
+
+	g_return_if_fail (props != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET_PROPERTIES (props));
+
+	if (time == props->priv->time)
+		return;
+
+	string = totem_time_to_string_text (time);
+	bacon_video_widget_properties_set_label (props, "duration", string);
+	g_free (string);
+
+	props->priv->time = time;
+}
+
 void
 bacon_video_widget_properties_update (BaconVideoWidgetProperties *props,
 				      BaconVideoWidget *bvw)
 {
 	GtkWidget *item;
 	GValue value = { 0, };
-	char *string;
 	gboolean has_type;
 
 	g_return_if_fail (props != NULL);
@@ -184,10 +204,8 @@ bacon_video_widget_properties_update (BaconVideoWidgetProperties *props,
 
 	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
 			BVW_INFO_DURATION, &value);
-	string = totem_time_to_string_text
-		(g_value_get_int (&value) * 1000);
-	bacon_video_widget_properties_set_label (props, "duration", string);
-	g_free (string);
+	bacon_video_widget_properties_from_time (props,
+			g_value_get_int (&value) * 1000);
 	g_value_unset (&value);
 
 	/* Video */
@@ -206,9 +224,10 @@ bacon_video_widget_properties_update (BaconVideoWidgetProperties *props,
 				  "dimensions", N_("%d x %d"));
 		UPDATE_FROM_STRING (BVW_INFO_VIDEO_CODEC, "vcodec");
 		UPDATE_FROM_INT (BVW_INFO_FPS, "framerate",
-				 N_("%d frames per second"));
+				 N_("%d frames per second"), _("N/A"));
 		UPDATE_FROM_INT (BVW_INFO_VIDEO_BITRATE, "video_bitrate",
-				 N_("%d kbps"));
+				 N_("%d kbps"), _("N/A"));
+		gtk_widget_show (item);
 	} else {
 		gtk_widget_hide (item);
 	}
@@ -224,7 +243,7 @@ bacon_video_widget_properties_update (BaconVideoWidgetProperties *props,
 	if (has_type != FALSE)
 	{
 		UPDATE_FROM_INT (BVW_INFO_AUDIO_BITRATE, "audio_bitrate",
-				 N_("%d kbps"));
+				 N_("%d kbps"), _("N/A"));
 		UPDATE_FROM_STRING (BVW_INFO_AUDIO_CODEC, "acodec");
 	}
 
