@@ -35,10 +35,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#ifdef HAVE_NCB
-#include <nautilus-burn-drive-selection.h>
-#endif /* HAVE_NCB */
-
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-preferences.h"
@@ -56,7 +52,7 @@ totem_action_info (char *reason, Totem *totem)
 	if (totem == NULL)
 		parent = NULL;
 	else
-		parent = totem->win;
+		parent = totem->prefs;
 
 	error_dialog =
 		gtk_message_dialog_new (GTK_WINDOW (parent),
@@ -218,36 +214,6 @@ deinterlace_changed_cb (GConfClient *client, guint cnxn_id,
 			G_CALLBACK (on_deinterlace1_activate), totem);
 }
 
-#ifdef HAVE_NCB
-static void
-on_combo_entry1_changed (NautilusBurnDriveSelection *bcs, char *device, Totem *totem)
-{
-	const char *str;
-
-	str = nautilus_burn_drive_selection_get_device (bcs);
-	if (str == NULL)
-		str = "";
-
-	gconf_client_set_string (totem->gc, GCONF_PREFIX"/mediadev", str, NULL);
-	bacon_video_widget_set_media_device
-		(BACON_VIDEO_WIDGET (totem->bvw), str);
-}
-#else /* HAVE_NCB */
-static void
-on_combo_entry1_changed (GtkEntry * entry, Totem * totem)
-{
-	const char *str;
-
-	str = gtk_entry_get_text (entry);
-	if (str == NULL)
-		str = "";
-
-	gconf_client_set_string (totem->gc, GCONF_PREFIX"/mediadev", str, NULL);
-	bacon_video_widget_set_media_device
-		(BACON_VIDEO_WIDGET (totem->bvw), str);
-}
-#endif /* HAVE_NCB */
-
 static void
 auto_resize_changed_cb (GConfClient *client, guint cnxn_id,
 		GConfEntry *entry, Totem *totem)
@@ -298,43 +264,6 @@ disable_save_to_disk_changed_cb (GConfClient *client, guint cnxn_id,
 	gtk_widget_set_sensitive (item, !locked);
 }
 
-static void
-mediadev_changed_cb (GConfClient *client, guint cnxn_id,
-		GConfEntry *entry, Totem *totem)
-{
-	GtkWidget *item;
-	char *mediadev;
-
-	item = glade_xml_get_widget (totem->xml, "tpw_device_combo");
-	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
-			on_combo_entry1_changed, totem);
-
-	mediadev = gconf_client_get_string (totem->gc,
-			GCONF_PREFIX"/mediadev", NULL);
-
-	if (mediadev == NULL || strcmp (mediadev, "") == 0) {
-		g_free (mediadev);
-		mediadev = g_strdup ("/dev/cdrom");
-	}
-
-#ifdef HAVE_NCB
-	nautilus_burn_drive_selection_set_device (NAUTILUS_BURN_DRIVE_SELECTION (item), mediadev);
-#else /* HAVE_NCB */
-	gtk_entry_set_text (GTK_ENTRY (item), mediadev);
-#endif /* HAVE_NCB */
-	bacon_video_widget_set_media_device
-		(BACON_VIDEO_WIDGET (totem->bvw), mediadev);
-	g_free (mediadev);
-
-#ifdef HAVE_NCB
-	g_signal_connect (G_OBJECT (item), "device-changed",
-			G_CALLBACK (on_combo_entry1_changed), totem);
-#else
-	g_signal_connect (G_OBJECT (item), "changed",
-			  G_CALLBACK (on_combo_entry1_changed), totem);
-#endif
-}
-
 static void
 connection_combobox_changed (GtkComboBox *combobox, Totem *totem)
 {
@@ -345,32 +274,6 @@ connection_combobox_changed (GtkComboBox *combobox, Totem *totem)
 		(BACON_VIDEO_WIDGET (totem->bvw), i);
 }
 
-static void
-on_button1_clicked (GtkButton *button, Totem *totem)
-{
-	GError *err = NULL;
-	char *path, *cmd, *filemanager;
-
-	path = g_build_path (G_DIR_SEPARATOR_S,
-			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
-
-	if ((g_getenv ("KDE_STARTUP_ENV") || g_getenv("KDE_FULL_SESSION")) && (filemanager = g_find_program_in_path("konqueror"))) {
-		cmd = g_strconcat (filemanager, " ", path, NULL);
-		g_free (filemanager);
-	} else {
-		cmd = g_strdup_printf ("nautilus --no-default-window -no-desktop %s", path);
-	}
-	g_free (path);
-
-	if (g_spawn_command_line_async (cmd, &err) == FALSE)
-	{
-		totem_action_error (_("Totem could not start the file manager."), err->message, totem);
-		g_error_free (err);
-	}
-
-	g_free (cmd);
-}
-
 static void
 visual_menu_changed (GtkComboBox *combobox, Totem *totem)
 {
@@ -408,21 +311,6 @@ visual_quality_menu_changed (GtkComboBox *combobox, Totem *totem)
 	bacon_video_widget_set_visuals_quality (totem->bvw, i);
 }
 
-GtkWidget *
-bacon_cd_selection_create (void)
-{
-	GtkWidget *widget;
-
-#ifdef HAVE_NCB
-	widget = nautilus_burn_drive_selection_new ();
-#else
-	widget = gtk_entry_new ();
-#endif
-	gtk_widget_show (widget);
-
-	return widget;
-}
-
 static void
 brightness_changed (GtkRange *range, Totem *totem)
 {
@@ -495,7 +383,29 @@ audio_out_menu_changed (GtkComboBox *combobox, Totem *totem)
 					"restarted."),
 				totem);
 	}
+}
 
+static void
+on_font_set (GtkFontButton * fb, Totem * totem)
+{
+	const gchar *font;
+
+	font = gtk_font_button_get_font_name (fb);
+	gconf_client_set_string (totem->gc, GCONF_PREFIX"/subtitle_font",
+				 font, NULL);
+}
+
+static void
+font_changed_cb (GConfClient *client, guint cnxn_id,
+		 GConfEntry *entry, Totem *totem)
+{
+	const gchar *font;
+	GtkWidget *item;
+
+	item = glade_xml_get_widget (totem->xml, "font_sel_button");
+	font = gconf_value_get_string (entry->value);
+	gtk_font_button_set_font_name (GTK_FONT_BUTTON (item), font);
+	bacon_video_widget_set_subtitle_font (totem->bvw, font);
 }
 
 void
@@ -504,7 +414,7 @@ totem_setup_preferences (Totem *totem)
 	GtkWidget *item, *menu;
 	gboolean show_visuals, auto_resize, is_local, deinterlace;
 	int connection_speed, i;
-	char *path, *visual, *mediadev;
+	char *path, *visual, *font;
 	GList *list, *l;
 	BaconVideoWidgetAudioOutType audio_out;
 
@@ -520,9 +430,6 @@ totem_setup_preferences (Totem *totem)
 	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/show_vfx",
 			(GConfClientNotifyFunc) show_vfx_changed_cb,
 			totem, NULL, NULL);
-	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/mediadev",
-			(GConfClientNotifyFunc) mediadev_changed_cb,
-			totem, NULL, NULL);
 	gconf_client_add_dir (totem->gc, "/desktop/gnome/lockdown",
 			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	gconf_client_notify_add (totem->gc,
@@ -549,40 +456,6 @@ totem_setup_preferences (Totem *totem)
 	bacon_video_widget_set_auto_resize
 		(BACON_VIDEO_WIDGET (totem->bvw), auto_resize);
 
-	/* Media device */
-	item = glade_xml_get_widget (totem->xml, "tpw_device_combo");
-	mediadev = g_strdup (gconf_client_get_string
-		(totem->gc, GCONF_PREFIX"/mediadev", NULL));
-	if (mediadev == NULL || (strcmp (mediadev, "") == 0)
-			|| (strcmp (mediadev, "auto") == 0))
-	{
-		g_free (mediadev);
-#ifdef HAVE_NCB
-		mediadev = g_strdup
-			(nautilus_burn_drive_selection_get_default_device
-				     (NAUTILUS_BURN_DRIVE_SELECTION (item)));
-#else /* HAVE_NCB */
-		mediadev = g_strdup ("/dev/cdrom");
-#endif /* HAVE_NCB */
-		gconf_client_set_string (totem->gc, GCONF_PREFIX"/mediadev",
-				mediadev, NULL);
-	}
-
-	bacon_video_widget_set_media_device
-		(BACON_VIDEO_WIDGET (totem->bvw), mediadev);
-
-#ifdef HAVE_NCB
-	g_signal_connect (G_OBJECT (item), "device-changed",
-			  G_CALLBACK (on_combo_entry1_changed), totem);
-	nautilus_burn_drive_selection_set_device
-		(NAUTILUS_BURN_DRIVE_SELECTION (item), mediadev);
-#else /* HAVE_NCB */
-	g_signal_connect (G_OBJECT (item), "changed",
-			  G_CALLBACK (on_combo_entry1_changed), totem);
-	gtk_entry_set_text (GTK_ENTRY (item), mediadev);
-#endif /* HAVE_NCB */
-	g_free (mediadev);
-
 	/* Connection Speed */
 	connection_speed = bacon_video_widget_get_connection_speed (totem->bvw);
 	item = glade_xml_get_widget (totem->xml, "tpw_speed_combobox");
@@ -591,9 +464,6 @@ totem_setup_preferences (Totem *totem)
 			G_CALLBACK (connection_combobox_changed), totem);
 
 	/* Proprietary plugins */
-	item = glade_xml_get_widget (totem->xml, "tpw_plugins_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (on_button1_clicked), totem);
 	path = g_build_path (G_DIR_SEPARATOR_S,
 			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
 	if (g_file_test (path, G_FILE_TEST_IS_DIR) == FALSE)
@@ -724,6 +594,22 @@ totem_setup_preferences (Totem *totem)
 			!gconf_client_get_bool (totem->gc,
 				"/desktop/gnome/lockdown/disable_save_to_disk",
 				NULL));
+
+	/* subtitle font selection */
+	item = glade_xml_get_widget (totem->xml, "font_sel_button");
+	g_signal_connect (item, "font-set", G_CALLBACK (on_font_set), totem);
+	gtk_font_button_set_title (GTK_FONT_BUTTON (item),
+				   _("Select Subtitle Font"));
+	font = gconf_client_get_string (totem->gc,
+		GCONF_PREFIX"/subtitle_font", NULL);
+	if (font && strcmp (font, "") != 0) {
+		gtk_font_button_set_font_name (GTK_FONT_BUTTON (item), font);
+		bacon_video_widget_set_subtitle_font (totem->bvw, font);
+	}
+	g_free (font);
+	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/subtitle_font",
+			(GConfClientNotifyFunc) font_changed_cb,
+			totem, NULL, NULL);
 }
 
 void
