@@ -116,6 +116,7 @@ static void show_controls (Totem *totem, gboolean was_fullscreen);
 static gboolean totem_is_fullscreen (Totem *totem);
 static void play_pause_set_label (Totem *totem, TotemStates state);
 static gboolean on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, Totem *totem);
+static void popup_timeout_remove (Totem *totem);
 
 static void
 long_action (void)
@@ -164,6 +165,8 @@ totem_action_exit (Totem *totem)
 {
 	GdkDisplay *display = NULL;
 
+	popup_timeout_remove (totem);
+
 	if (gtk_main_level () > 0)
 		gtk_main_quit ();
 
@@ -232,7 +235,7 @@ static void
 play_pause_set_label (Totem *totem, TotemStates state)
 {
 	GtkWidget *image;
-	const char *id;
+	const char *id, *tip;
 
 	if (state == totem->state)
 		return;
@@ -243,11 +246,13 @@ play_pause_set_label (Totem *totem, TotemStates state)
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
 				_("Playing"));
 		id = GTK_STOCK_MEDIA_PAUSE;
+		tip = N_("Pause");
 		break;
 	case STATE_PAUSED:
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
 				_("Paused"));
 		id = GTK_STOCK_MEDIA_PLAY;
+		tip = N_("Play");
 		break;
 	case STATE_STOPPED:
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
@@ -255,11 +260,17 @@ play_pause_set_label (Totem *totem, TotemStates state)
 		totem_statusbar_set_time_and_length
 			(TOTEM_STATUSBAR (totem->statusbar), 0, 0);
 		id = GTK_STOCK_MEDIA_PLAY;
+		tip = N_("Play");
 		break;
 	default:
 		return;
 	}
 
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (totem->tooltip),
+			totem->pp_button, _(tip), NULL);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (totem->tooltip),
+			totem->fs_pp_button, _(tip), NULL);
+
 	image = glade_xml_get_widget (totem->xml,
 			"tmw_play_pause_button_image");
 	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_BUTTON);
@@ -338,6 +349,8 @@ totem_action_seek (Totem *totem, double pos)
 
 	if (totem->mrl == NULL)
 		return;
+	if (bacon_video_widget_is_seekable (totem->bvw) == FALSE)
+		return;
 
 	retval = bacon_video_widget_seek (totem->bvw, pos, &err);
 
@@ -369,6 +382,7 @@ totem_action_load_media (Totem *totem, MediaType type)
 {
 	char **mrls;
 	char *msg;
+	gboolean retval;
 
 	if (bacon_video_widget_can_play (totem->bvw, type) == FALSE)
 	{
@@ -387,10 +401,10 @@ totem_action_load_media (Totem *totem, MediaType type)
 		return FALSE;
 	}
 
-	totem_action_open_files (totem, mrls);
+	retval = totem_action_open_files (totem, mrls);
 	g_strfreev (mrls);
 
-	return TRUE;
+	return retval;
 }
 
 static gboolean
@@ -421,17 +435,19 @@ totem_action_load_media_device (Totem *totem, const char *device)
 			{
 				char *s;
 				s = totem_action_open_dialog (totem, url, FALSE);
+				retval = (s != NULL);
 				g_free (s);
 			}
-			retval = TRUE;
 			break;
 		case MEDIA_TYPE_DVD:
 		case MEDIA_TYPE_VCD:
 		case MEDIA_TYPE_CDDA:
 			bacon_video_widget_set_media_device
 				(BACON_VIDEO_WIDGET (totem->bvw), device_path);
-			totem_action_load_media (totem, type);
-			retval = TRUE;
+			retval = totem_action_load_media (totem, type);
+			if (retval == FALSE) {
+				totem_action_set_mrl_and_play (totem, NULL);
+			}
 			break;
 		default:
 			g_assert_not_reached ();
@@ -452,7 +468,7 @@ totem_action_play_media_device (Totem *totem, const char *device)
 		mrl = totem_playlist_get_current_mrl (totem->playlist);
 		totem_action_set_mrl_and_play (totem, mrl);
 		g_free (mrl);
-	}
+	} 
 }
 
 void
@@ -518,22 +534,34 @@ totem_action_pause (Totem *totem)
 	}
 }
 
-void
-totem_action_fullscreen_toggle (Totem *totem)
+static gboolean
+window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
+		Totem *totem)
 {
-	if (totem_is_fullscreen (totem) != FALSE)
-	{
+	if (event->changed_mask != GDK_WINDOW_STATE_FULLSCREEN) {
+		return FALSE;
+	}
+
+	if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) {
+		totem_action_save_size (totem);
+		update_fullscreen_size (totem);
+		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
+		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
+
+		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+			totem_scrsaver_disable (totem->scr);
+
+		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
+		show_controls (totem, FALSE);
+	} else {
 		GtkWidget *item;
 
 		popup_hide (totem);
 		bacon_video_widget_set_fullscreen (totem->bvw, FALSE);
-		gtk_window_unfullscreen (GTK_WINDOW(totem->win));
 		bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
 
 		totem_scrsaver_enable (totem->scr);
 
-		totem->controls_visibility = TOTEM_CONTROLS_VISIBLE;
-
 		item = glade_xml_get_widget
 			(totem->xml, "tmw_show_controls_menu_item");
 
@@ -546,18 +574,18 @@ totem_action_fullscreen_toggle (Totem *totem)
 			totem->controls_visibility = TOTEM_CONTROLS_HIDDEN;
 			show_controls (totem, TRUE);
 		}
+	}
+
+	return FALSE;
+}
+
+void
+totem_action_fullscreen_toggle (Totem *totem)
+{
+	if (totem_is_fullscreen (totem) != FALSE) {
+		gtk_window_unfullscreen (GTK_WINDOW (totem->win));
 	} else {
-		totem_action_save_size (totem);
-		update_fullscreen_size (totem);
-		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
-		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
-		gtk_window_fullscreen (GTK_WINDOW(totem->win));
-
-		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
-			totem_scrsaver_disable (totem->scr);
-
-		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
-		show_controls (totem, FALSE);
+		gtk_window_fullscreen (GTK_WINDOW (totem->win));
 	}
 }
 
@@ -856,10 +884,10 @@ totem_action_seek_relative (Totem *totem, int off_sec)
 	GError *err = NULL;
 	gint64 off_msec, oldsec, sec;
 
-	if (bacon_video_widget_is_seekable (totem->bvw) == FALSE)
-		return;
 	if (totem->mrl == NULL)
 		return;
+	if (bacon_video_widget_is_seekable (totem->bvw) == FALSE)
+		return;
 
 	off_msec = off_sec * 1000;
 	oldsec = bacon_video_widget_get_current_time (totem->bvw);
@@ -1525,70 +1553,37 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 char *
 totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
 {
-	GtkWidget *fs;
-	int response;
-	char *path_new = NULL;
+	GSList *filenames;
+	char *new_path;
+	gboolean playlist_modified;
 
-	fs = gtk_file_chooser_dialog_new (_("Select Files"),
-			GTK_WINDOW (totem->win), GTK_FILE_CHOOSER_ACTION_OPEN,
-			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-			NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
+	filenames = totem_add_files (GTK_WINDOW (totem->win),
+			path, &new_path);
 
-	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fs), TRUE);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+	if (filenames == NULL)
+		return NULL;
 
-	if (path != NULL)
-	{
-		gtk_file_chooser_set_current_folder_uri
-			(GTK_FILE_CHOOSER (fs), path);
-	}
-
-	while (1)
-	{
-		GSList *filenames;
-		gboolean playlist_modified;
-
-		response = gtk_dialog_run (GTK_DIALOG (fs));
-		if (response != GTK_RESPONSE_ACCEPT)
-			break;
-
-		filenames = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (fs));
-		if (filenames == NULL)
-			continue;
-
-		playlist_modified = totem_action_open_files_list (totem,
-				filenames);
-		if (playlist_modified == FALSE)
-		{
-			g_slist_foreach (filenames, (GFunc) g_free, NULL);
-			g_slist_free (filenames);
-			continue;
-		}
-
-		/* Hide the selection widget only if playlist is modified */
-		gtk_widget_hide (fs);
-
-		/* Save the path */
-		if (filenames->data != NULL)
-			path_new = g_path_get_dirname (filenames->data);
+	playlist_modified = totem_action_open_files_list (totem,
+			filenames);
 
+	if (playlist_modified == FALSE) {
 		g_slist_foreach (filenames, (GFunc) g_free, NULL);
 		g_slist_free (filenames);
-
-		if (play != FALSE) {
-			char *mrl;
-
-			mrl = totem_playlist_get_current_mrl (totem->playlist);
-			totem_action_set_mrl_and_play (totem, mrl);
-			g_free (mrl);
-		}
-		break;
+		return NULL;
 	}
 
-	gtk_widget_destroy (fs);
-	return path_new;
+	g_slist_foreach (filenames, (GFunc) g_free, NULL);
+	g_slist_free (filenames);
+
+	if (play != FALSE) {
+		char *mrl;
+
+		mrl = totem_playlist_get_current_mrl (totem->playlist);
+		totem_action_set_mrl_and_play (totem, mrl);
+		g_free (mrl);
+	}
+
+	return new_path;
 }
 
 static void
@@ -2211,10 +2206,9 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 	case TOTEM_REMOTE_COMMAND_REPLACE:
 		g_assert (url != NULL);
 		totem_playlist_clear (totem->playlist);
-		if (g_str_has_prefix (url, "dvd:"))
-		{
+		if (strcmp (url, "dvd:") == 0) {
 			totem_action_play_media (totem, MEDIA_TYPE_DVD);
-		} else if (g_str_has_prefix (url, "vcd:") != FALSE) {
+		} else if (strcmp (url, "vcd:") == 0) {
 			totem_action_play_media (totem, MEDIA_TYPE_VCD);
 		} else if (g_str_has_prefix (url, "cd:") != FALSE) {
 			totem_action_play_media (totem, MEDIA_TYPE_CDDA);
@@ -2223,7 +2217,7 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		} else if (totem_playlist_add_mrl (totem->playlist,
 					url, NULL) != FALSE) {
 			totem_action_add_recent (totem, url);
-		}	
+		}
 		break;
 	case TOTEM_REMOTE_COMMAND_SHOW:
 		gtk_window_present (GTK_WINDOW (totem->win));
@@ -2296,6 +2290,12 @@ playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 	g_free (mrl);
 }
 
+static void
+item_activated_cb (GtkWidget *playlist, Totem *totem)
+{
+	totem_action_seek (totem, 0);
+}
+
 static void
 current_removed_cb (GtkWidget *playlist, Totem *totem)
 {
@@ -2416,6 +2416,13 @@ theme_changed_cb (GtkIconTheme *icon_theme, Totem *totem)
 	move_popups (totem);
 }
 
+static void
+popup_timeout_add(Totem* totem)
+{
+	totem->popup_timeout = g_timeout_add (FULLSCREEN_POPUP_TIMEOUT,
+		(GSourceFunc) popup_hide, totem);
+}
+
 static void
 popup_timeout_remove (Totem *totem)
 {
@@ -2449,9 +2456,7 @@ static void
 on_mouse_click_fullscreen (GtkWidget *widget, Totem *totem)
 {
 	popup_timeout_remove (totem);
-
-	totem->popup_timeout = g_timeout_add (FULLSCREEN_POPUP_TIMEOUT,
-		(GSourceFunc) popup_hide, totem);
+	popup_timeout_add (totem);
 }
 
 static gboolean
@@ -2480,8 +2485,7 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 	gtk_widget_show_all (totem->control_popup);
 	bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
 
-	totem->popup_timeout = g_timeout_add (FULLSCREEN_POPUP_TIMEOUT,
-			(GSourceFunc) popup_hide, totem);
+	popup_timeout_add (totem);
 	totem->popup_in_progress = FALSE;
 
 	return FALSE;
@@ -2565,6 +2569,11 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 	case GDK_F:
 		totem_action_fullscreen_toggle (totem);
 		break;
+	case GDK_g:
+	case GDK_G:
+		bacon_video_widget_dvd_event (totem->bvw,
+				BVW_DVD_NEXT_ANGLE);
+		break;
 	case GDK_h:
 	case GDK_H:
 		totem_action_toggle_controls (totem);
@@ -2790,6 +2799,8 @@ update_media_menu_items (Totem *totem)
 	totem_main_set_sensitivity ("tmw_dvd_audio_menu_item", playing);
 	totem_main_set_sensitivity ("tmw_dvd_angle_menu_item", playing);
 	totem_main_set_sensitivity ("tmw_dvd_chapter_menu_item", playing);
+	/* FIXME we should only show that if we have multiple angles */
+	totem_main_set_sensitivity ("tmw_next_angle_menu_item", playing);
 
 	playing = totem_is_media (totem->mrl);
 
@@ -3029,6 +3040,8 @@ totem_callback_connect (Totem *totem)
 	g_object_notify (G_OBJECT (totem->win), "is-active");
 	g_signal_connect_swapped (G_OBJECT (totem->win), "notify",
 			G_CALLBACK (popup_hide), totem);
+	g_signal_connect (G_OBJECT (totem->win), "window-state-event",
+			G_CALLBACK (window_state_event_cb), totem);
 
 	/* Screen size and Theme changes */
 	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
@@ -3124,20 +3137,6 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (vol_cb), totem);
 
 	/* Playlist */
-	g_signal_connect (G_OBJECT (totem->playlist),
-			"changed", G_CALLBACK (playlist_changed_cb),
-			totem);
-	g_signal_connect (G_OBJECT (totem->playlist),
-			"current-removed", G_CALLBACK (current_removed_cb),
-			totem);
-	g_signal_connect (G_OBJECT (totem->playlist),
-			"repeat-toggled",
-			G_CALLBACK (playlist_repeat_toggle_cb),
-			totem);
-	g_signal_connect (G_OBJECT (totem->playlist),
-			"shuffle-toggled",
-			G_CALLBACK (playlist_shuffle_toggle_cb),
-			totem);
 
 	/* DVD menu callbacks */
 	item = glade_xml_get_widget (totem->xml, "tmw_dvd_root_menu_item");
@@ -3196,8 +3195,25 @@ playlist_widget_setup (Totem *totem)
 
 	gtk_widget_show_all (GTK_WIDGET (totem->playlist));
 
-	g_signal_connect(totem->playlist, "active-name-changed",
+	g_signal_connect (G_OBJECT (totem->playlist), "active-name-changed",
 			G_CALLBACK (on_playlist_change_name), totem);
+	g_signal_connect (G_OBJECT (totem->playlist), "item-activated",
+			G_CALLBACK (item_activated_cb), totem);
+	g_signal_connect (G_OBJECT (totem->playlist),
+			"changed", G_CALLBACK (playlist_changed_cb),
+			totem);
+	g_signal_connect (G_OBJECT (totem->playlist),
+			"current-removed", G_CALLBACK (current_removed_cb),
+			totem);
+	g_signal_connect (G_OBJECT (totem->playlist),
+			"repeat-toggled",
+			G_CALLBACK (playlist_repeat_toggle_cb),
+			totem);
+	g_signal_connect (G_OBJECT (totem->playlist),
+			"shuffle-toggled",
+			G_CALLBACK (playlist_shuffle_toggle_cb),
+			totem);
+
 }
 
 static void
@@ -3450,6 +3466,7 @@ main (int argc, char **argv)
 		(totem->xml, "tcw_volume_hscale");
 	totem->fs_voladj = gtk_range_get_adjustment
 		(GTK_RANGE (totem->fs_volume));
+	totem->tooltip = gtk_tooltips_new ();
 	g_object_set_data (G_OBJECT (totem->fs_volume), "fs", GINT_TO_POINTER (1));
 	totem->volume_first_time = 1;
 	totem->fs_pp_button = glade_xml_get_widget
@@ -3465,6 +3482,7 @@ main (int argc, char **argv)
 	totem_session_setup (totem, argv);
 	totem_setup_recent (totem);
 	totem_setup_file_monitoring (totem);
+	totem_setup_file_filters ();
 	totem_setup_play_disc (totem);
 	totem_callback_connect (totem);
 	totem_setup_window (totem);
